package org.blue.club.controller;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import org.blue.club.annotation.Auth;
import org.blue.club.annotation.DescriptionType;
import org.blue.club.annotation.PermissionDescription;
import org.blue.club.entities.dto.club.req.*;
import org.blue.club.entities.dto.user.User;
import org.blue.club.entities.vo.data.ClubVo;
import org.blue.club.services.ClubServices;
import org.mmga.spring.boot.starter.entities.PagerData;
import org.mmga.spring.boot.starter.entities.Result;
import org.springframework.web.bind.annotation.*;

@RestController
@Tag(name = "社团相关接口")
@RequestMapping("/api/club")
@RequiredArgsConstructor
public class ClubController {
    private final ClubServices clubServices;

    @GetMapping("/")
    @Operation(description = "分页获取社团")
    public Result<PagerData<ClubVo>> getClubs(@Schema(description = "分页数据第几页") @RequestParam Integer page, @Schema(description = "分页数据每页数据量") @RequestParam Integer num) {
        return clubServices.getClubs(page, num);
    }

    @Operation(description = "创建社团")
    @Auth(permissions = {@PermissionDescription(3)})
    @PutMapping("/create")
    public Result<ClubVo> createClub(@RequestBody CreateClubRequest request) {
        return clubServices.createClub(request);
    }

    @Operation(description = "向社团添加用户")
    @PutMapping("/user/add")
    public Result<Boolean> clubAddUser(@Auth(permissions = {@PermissionDescription(3), @PermissionDescription(value = 2, type = DescriptionType.CLUB)}) User user, @RequestBody ClubAddUserRequest request) {
        return clubServices.clubAddUser(user, request);
    }

    @Operation(description = "分页列出社团的所有用户")
    @GetMapping("/user/list")
    public Result<PagerData<User>> clubGetUsers(@Schema(description = "分页数据第几页") @RequestParam Integer page, @Schema(description = "分页数据每页数据量") @RequestParam Integer num, @Schema(description = "社团ID") @RequestParam Long clubId) {
        return clubServices.clubGetUsers(page, num, clubId);
    }

    @Operation(description = "从社团中删除用户")
    @DeleteMapping("/user/remove")
    public Result<Boolean> clubRemoveUser(@Auth(permissions = {@PermissionDescription(3), @PermissionDescription(value = 2, type = DescriptionType.CLUB)}) User user, @Schema(description = "社团ID") @RequestParam long clubId, @Schema(description = "需要删除的用户ID") @RequestParam long userId) {
        return clubServices.clubRemoveUser(user, clubId, userId);
    }

    @Operation(description = "修改社团用户权限")
    @PutMapping("/user/auth")
    public Result<Boolean> clubChangeUserAuth(@Auth(permissions = {@PermissionDescription(3), @PermissionDescription(value = 2, type = DescriptionType.CLUB)}) User user, @RequestBody ClubChangeUserAuthRequest request) {
        return clubServices.clubChangeUserAuth(user, request);
    }

    @PutMapping("/avatar")
    @Operation(description = "修改社团头像")
    public Result<Boolean> changeAvatar(@RequestParam("code") @Schema(description = "修改头像操作码，可以通过将图片文件上传至([POST] /api/avatar/upload)接口获取") String avatarOperationCode, @Schema(description = "被修改的社团ID") @RequestParam long clubId, @Auth(permissions = {@PermissionDescription(3), @PermissionDescription(value = 1, type = DescriptionType.CLUB)}) User user) {
        return clubServices.changeAvatar(avatarOperationCode, clubId, user);
    }

    @PutMapping("/name")
    @Operation(description = "修改社团名称")
    @Auth(permissions = {@PermissionDescription(3)})
    public Result<Boolean> changeName(@RequestBody ChangeClubNameRequest request) {
        return clubServices.changeName(request);
    }

    @PutMapping("/commit")
    @Operation(description = "修改社团名称")
    public Result<Boolean> changeCommit(@RequestBody ChangeClubCommitRequest request, @Auth(permissions = {@PermissionDescription(3), @PermissionDescription(value = 1, type = DescriptionType.CLUB)}) User user) {
        return clubServices.changeCommit(request, user);
    }

    @GetMapping("/{id}")
    @Operation(description = "获取社团")
    public Result<ClubVo> getClub(@Schema(description = "社团ID") @PathVariable("id") long id) {
        return clubServices.getClub(id);
    }
}
