package org.blue.club.controller;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.blue.club.annotation.Auth;
import org.blue.club.annotation.PermissionDescription;
import org.blue.club.entities.dto.user.User;
import org.blue.club.entities.dto.user.req.*;
import org.blue.club.entities.dto.user.resp.VerifyCodeResponse;
import org.blue.club.services.UserServices;
import org.mmga.spring.boot.starter.annotation.AuthMapping;
import org.mmga.spring.boot.starter.entities.PagerData;
import org.mmga.spring.boot.starter.entities.Result;
import org.springframework.web.bind.annotation.*;

@Slf4j
@RestController
@RequestMapping("/api/user")
@RequiredArgsConstructor
@Tag(name = "用户相关接口")
public class UserController {
    private final UserServices userServices;

    @PostMapping("/login")
    @AuthMapping
    @Operation(description = "用户登录", responses = {@ApiResponse(description = "返回是否登录成功", responseCode = "200")})
    public Result<User> login(@RequestBody LoginRequest user) {
        return userServices.login(user);
    }

    @AuthMapping
    @PutMapping("/create")
    @Operation(description = "创建用户", responses = {@ApiResponse(description = "返回创建后的用户")})
    public Result<User> createUser(@RequestBody RegisterRequest user) {
        return userServices.createUser(user);
    }

    @GetMapping("/info")
    @Operation(description = "获取用户信息")
    public Result<User> getUserInfo(@Auth User user) {
        return Result.success(user);
    }

    @GetMapping("/all")
    @Operation(description = "获取所有用户信息（分页）")
    @Auth(permissions = {@PermissionDescription(3), @PermissionDescription(4)})
    public Result<PagerData<User>> getAllUserInfo(@RequestParam("num") int num, @RequestParam("page") int page) {
        return userServices.getAllUserInfo(num, page);
    }

    @PutMapping("/rename")
    @Operation(description = "修改用户名")
    @Auth
    public Result<Boolean> changeUsername(@RequestBody UserRenameRequest renameVo, @Auth User user) {
        return userServices.changeUsername(renameVo, user);
    }

    @PutMapping("/password")
    @Operation(description = "修改密码")
    public Result<Boolean> changePassword(@RequestBody UserChangePasswordRequest changePasswordVo, @Auth User user) {
        return userServices.changePassword(changePasswordVo, user);
    }

    @PutMapping("/auth")
    @Operation(description = "修改用户权限组")
    @Auth(permissions = {@PermissionDescription(3), @PermissionDescription(4)})
    public Result<Boolean> changeAuth(@RequestBody UserChangeAuthRequest userChangeAuthVo) {
        return userServices.changeAuth(userChangeAuthVo);
    }

    @PutMapping("/avatar")
    @Operation(description = "修改用户头像")
    public Result<Boolean> changeAvatar(@RequestParam("code") @Schema(description = "修改头像操作码，可以通过将图片文件上传至([POST] /api/avatar/upload)接口获取") String avatarOperationCode, @Schema(description = "被修改的用户ID", requiredMode = Schema.RequiredMode.NOT_REQUIRED) @RequestParam(defaultValue = "-1") long userId, @Auth User user) {
        return userServices.changeAvatar(avatarOperationCode, userId, user);
    }

    @GetMapping("/verify")
    @Operation(description = "获取验证码")
    public Result<VerifyCodeResponse> generatorVerifyCode() {
        return userServices.getVerifyCode();
    }

    @GetMapping("/info/{id}")
    @Operation(description = "获取简略用户信息")
    public Result<User> getSimpleInfo(@PathVariable("id") int userId) {
        return userServices.getUserInfo(userId);
    }
}
