package org.blue.club.services;

import jakarta.servlet.ServletOutputStream;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.tomcat.util.buf.HexUtils;
import org.blue.club.dao.redis.AvatarOperationDao;
import org.blue.club.entities.vo.data.AvatarTmpVo;
import org.blue.club.exceptions.ServerException;
import org.blue.club.utils.iop.FileUtils;
import org.mmga.spring.boot.starter.entities.Result;
import org.mmga.spring.boot.starter.utils.RandomUtils;
import org.springframework.http.ContentDisposition;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.util.StreamUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.security.DigestInputStream;
import java.security.MessageDigest;
import java.util.Date;
import java.util.Optional;

@Service
@RequiredArgsConstructor
public class AvatarServices {
    private final RandomUtils randomUtils;
    private final File tmpFolder;
    private final File avatarFolder;
    private final AvatarOperationDao avatarOperationDao;
    private final FileUtils fileUtils;

    @SneakyThrows
    public Result<String> changeAvatar(MultipartFile file) {
        String s = randomUtils.generatorRandomFileName(32);
        DigestInputStream digestInputStream = new DigestInputStream(file.getInputStream(), DigestUtils.getSha1Digest());
        File tmpFile = new File(tmpFolder, s);
        try (FileOutputStream fos = new FileOutputStream(tmpFile)) {
            digestInputStream.transferTo(fos);
        }
        MessageDigest messageDigest = digestInputStream.getMessageDigest();
        String sha1Hex = HexUtils.toHexString(messageDigest.digest());
        avatarOperationDao.save(new AvatarTmpVo(s, sha1Hex, new Date()));
        return Result.success("上传成功", s);
    }

    @SneakyThrows
    public void getAvatar(HttpServletResponse response, String sha1) {
        File targetFile = new File(avatarFolder, sha1);
        if (!targetFile.getAbsolutePath().startsWith(avatarFolder.getAbsolutePath())) {
            Result.failed(HttpStatus.NOT_FOUND, "文件不存在！").writeToResponse(response);
            return;
        }
        if (!targetFile.exists()) {
            Result.failed(HttpStatus.NOT_FOUND, "文件不存在！").writeToResponse(response);
            return;
        }
        try (FileInputStream fileInputStream = new FileInputStream(targetFile)) {
            response.setContentType("image/png");
            response.setHeader("Content-Disposition", ContentDisposition.attachment().filename(sha1 + ".png").build().toString());
            ServletOutputStream outputStream = response.getOutputStream();
            StreamUtils.copy(fileInputStream, outputStream);
        }
    }

    public Optional<String> permanentSaveAvatar(String avatarOperationCode) {
        Optional<AvatarTmpVo> byId = avatarOperationDao.findById(avatarOperationCode);
        if (byId.isEmpty()) return Optional.empty();
        AvatarTmpVo avatarTmpVo = byId.get();
        File tmpFile = new File(tmpFolder, avatarOperationCode);
        String sha1 = avatarTmpVo.sha1();
        File targetFile = new File(avatarFolder, sha1);
        changeAvatarFile(tmpFile, targetFile);
        avatarOperationDao.deleteById(avatarOperationCode);
        return Optional.of(sha1);
    }

    public void changeAvatarFile(File tmpFile, File targetFile) {
        if (targetFile.exists()) {
            fileUtils.tryDeleteOrDeleteOnExit(tmpFile);
            return;
        }
        if (!tmpFile.renameTo(targetFile))
            throw new ServerException(Result.failed(HttpStatus.INTERNAL_SERVER_ERROR, "移动头像文件失败！请联系管理员处理"));
    }
}
