package org.blue.club.services;

import com.mybatisflex.core.paginate.Page;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.blue.club.dao.UserDao;
import org.blue.club.dao.redis.VerifyDao;
import org.blue.club.entities.dto.user.User;
import org.blue.club.entities.dto.user.req.*;
import org.blue.club.entities.dto.user.resp.VerifyCodeResponse;
import org.blue.club.entities.vo.VerifyVo;
import org.blue.club.entities.vo.data.UserVo;
import org.blue.club.utils.iop.VerifyCodeUtils;
import org.mmga.spring.boot.starter.entities.PagerData;
import org.mmga.spring.boot.starter.entities.Result;
import org.mmga.spring.boot.starter.exception.AuthorizationException;
import org.mmga.spring.boot.starter.properties.Env;
import org.mmga.spring.boot.starter.properties.MainProperties;
import org.mmga.spring.boot.starter.utils.VoUtils;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Optional;

import static com.mybatisflex.core.query.QueryMethods.select;
import static org.blue.club.entities.vo.data.table.UserVoTableDef.USER_VO;

@Slf4j
@Service
@RequiredArgsConstructor
public class UserServices {
    private final UserDao userDao;
    private final VerifyDao verifyDao;
    private final VoUtils voUtils;
    private final VerifyCodeUtils verifyCodeUtils;
    private final MainProperties mainProperties;
    private final AvatarServices avatarServices;

    private boolean isWrongVerifyCode(String key, String code) {
        if (mainProperties.getEnv().equals(Env.DEV)) return false;
        Optional<VerifyVo> targetCode = verifyDao.findById(key);
        if (targetCode.isEmpty()) {
            return true;
        }
        verifyDao.deleteById(key);
        return !targetCode.get().answer().equals(code);
    }

    @Transactional
    public Result<User> login(LoginRequest loginRequest) {
        if (this.isWrongVerifyCode(loginRequest.key(), loginRequest.code())) {
            return Result.failed(HttpStatus.FORBIDDEN, "错误的验证码");
        }
        UserVo userVo = userDao.selectOneWithRelationsByCondition(USER_VO.NAME.eq(loginRequest.username()).and(USER_VO.PASSWORD.eq(loginRequest.sha1HexPassword())));
        if (userVo == null) return Result.failed(HttpStatus.NOT_FOUND, "用户不存在！");
        User user = voUtils.vo2DtoSafe(userVo, User.class);
        return Result.success(user);
    }

    @Transactional
    public Result<User> createUser(RegisterRequest request) {
        if (this.isWrongVerifyCode(request.key(), request.code())) {
            return Result.failed(HttpStatus.FORBIDDEN, "错误的验证码");
        }
        if (userDao.selectCountByCondition(USER_VO.NAME.eq(request.username())) > 0) {
            return Result.failed(HttpStatus.CONFLICT, "用户已存在");
        }
        UserVo userVo = new UserVo();
        userVo.setName(request.username());
        userVo.setPassword(request.sha1HexPassword());
        userVo.setAuth(request.auth());
        userDao.insert(userVo);
        System.out.println(userVo);
        UserVo insertedUserData = userDao.selectOneWithRelationsById(userVo.getId());
        return Result.success(voUtils.vo2DtoSafe(insertedUserData, User.class));
    }

    public Result<PagerData<User>> getAllUserInfo(int num, int page) {
        Page<UserVo> userVoPage = userDao.paginateWithRelations(page, num, select(USER_VO.ALL_COLUMNS).from(USER_VO));
        long totalRow = userVoPage.getTotalRow();
        List<UserVo> records = userVoPage.getRecords();
        return Result.success(new PagerData<>(totalRow, records.stream().map(e -> voUtils.vo2DtoSafe(e, User.class)).toList()));
    }

    public Result<Boolean> changeUsername(UserRenameRequest renameVo, User user) {
        Long id = renameVo.getId();
        Long operatorId = user.getId();
        if (id == null) id = operatorId;
        if (!operatorId.equals(id) && !user.hasPermission(4L))
            throw new AuthorizationException(Result.failed(HttpStatus.UNAUTHORIZED, "权限不足！"));
        String s = renameVo.getNewName();
        if (userDao.selectCountByCondition(USER_VO.NAME.eq(s)) > 0)
            return Result.failed(HttpStatus.CONFLICT, "用户名已存在！");
        UserVo userVo = new UserVo();
        userVo.setId(id);
        userVo.setName(s);
        userDao.update(userVo);
        return Result.success(true);
    }

    public Result<Boolean> changePassword(UserChangePasswordRequest changePasswordVo, User user) {
        UserChangePasswordRequest shaHex = changePasswordVo.sha1HexPassword();
        String newPassword = shaHex.getNewPassword();
        String oldPassword = shaHex.getOldPassword();
        Long id = changePasswordVo.getId();
        UserVo newUserVo = new UserVo();
        newUserVo.setId(id);
        newUserVo.setPassword(newPassword);
        if (user.hasPermission(4L)) {
            userDao.update(newUserVo);
            return Result.success(true);
        }
        Long operatorUserId = user.getId();
        if (oldPassword == null || !id.equals(operatorUserId))
            throw new AuthorizationException(Result.failed(HttpStatus.UNAUTHORIZED, "权限不足！"));
        if (userDao.selectCountByCondition(USER_VO.ID.eq(operatorUserId)) <= 0)
            return Result.failed(HttpStatus.NOT_FOUND, "旧密码不匹配");
        userDao.update(newUserVo);
        return Result.success(true);
    }

    public Result<Boolean> changeAuth(UserChangeAuthRequest userChangeAuthVo) {
        UserVo userVo = new UserVo();
        userVo.setId(userChangeAuthVo.id());
        userVo.setAuth(userChangeAuthVo.authId());
        userDao.update(userVo);
        return Result.success(true);
    }


    public Result<Boolean> changeAvatar(String avatarOperationCode, long userId, User user) {
        long operatorUserId = user.getId();
        if (userId == -1) {
            userId = operatorUserId;
        }
        if (userId != operatorUserId && !user.hasSuperAdminPermission())
            throw new AuthorizationException(Result.failed(HttpStatus.FORBIDDEN, "权限不足！"));
        Optional<String> result = avatarServices.permanentSaveAvatar(avatarOperationCode);
        if (result.isEmpty()) return Result.failed(HttpStatus.NOT_FOUND, "操作码错误或过期");
        UserVo userVo = new UserVo();
        userVo.setId(userId);
        userVo.setAvatar(result.get());
        userDao.update(userVo);
        return Result.success(true);
    }


    public Result<VerifyCodeResponse> getVerifyCode() {
        VerifyCodeResponse verifyCodeResponse = verifyCodeUtils.generateVerifyCode();
        return Result.success(verifyCodeResponse);
    }


    public Result<User> getUserInfo(int userId) {
        UserVo userVo = userDao.selectOneWithRelationsById(userId);
        if (userVo == null) return Result.failed("用户不存在！");
        User user = voUtils.vo2DtoSafe(userVo, User.class);
        return Result.success(user);
    }
}
